# -*- coding: utf-8 -*-
# @project: dujournal
# @Author：dyz
# @date：2024/1/16 14:41
# 预警期刊
import re
from typing import Union, List

import requests
from parsel import Selector
from schema.zky import ZkyWarnHit


class ZkyWarn:
    def __init__(self):
        self.start_year = 2019  # 预警期刊开始年份
        self.zky_warn_url = 'https://earlywarning.fenqubiao.com/_sidebar.md'
        self.max_url = None

    def get_new_year(self, year: int = None, all_year=False) -> Union[str, List]:
        """取预警期刊的最新年份"""
        year = int(year) if year else None
        resp = requests.get(self.zky_warn_url)
        resp.encoding = resp.apparent_encoding
        html = resp.text
        max_year = 0
        data_list = []
        for i in html.split('\n'):
            i = i.strip('').lstrip('* ').strip()
            if '年预警名单]' in i:
                _year = int(re.findall('\[(\\d{4})年预警名单]', i)[0])
                url = re.findall('年预警名单]\((/zh-cn/early-warning-journal-list-\d{4}.md)', i)[0]
                href = f'https://earlywarning.fenqubiao.com{url}'
                data_list.append(href)
                if year and _year == year:
                    return href
                if _year > max_year:
                    max_year = _year
                    self.max_url = href
        if all_year:
            return data_list
        return self.max_url

    def all(self):
        """所有年份数据"""
        href_list = self.get_new_year(all_year=True)
        for href in href_list:
            resp = requests.get(href)
            resp.encoding = resp.apparent_encoding
            yield self.parse(resp.text)

    @staticmethod
    def parse(resp) -> List[ZkyWarnHit]:
        doc = Selector(resp)
        year = int(re.findall('(\\d{4})年《国际期刊预警名单', resp)[0])
        tr_list = doc.css('tbody > tr')
        data_list = []
        if year < 2024:
            for tr in tr_list:
                td_list = tr.xpath('./td')
                name = td_list[-2].xpath('./text()').get('').strip()
                level = td_list[-1].xpath('./text()').get('').strip()
                item = ZkyWarnHit(ename=name, level=level, year=year)
                data_list.append(item)
        else:
            for tr in tr_list:
                td_list = tr.xpath('./td')
                name = td_list[-3].xpath('./text()').get('').strip()
                issn = td_list[-2].xpath('./text()').get('').strip()
                level = td_list[-1].xpath('./text()').get('').strip()
                item = ZkyWarnHit(ename=name, issn=issn, level=level, year=year)
                data_list.append(item)
        return data_list

    def get_zky_warn_list(self, year=None):
        """
        获取预警名单
        year: 预警年份
        :return: List[ZkyWarnHit]
        """
        if year:
            href = self.get_new_year(year)
        else:
            href = self.get_new_year()
        resp = requests.get(href)
        resp.encoding = resp.apparent_encoding
        return self.parse(resp.text)

    def run(self):
        for data in self.all():
            print(data)


if __name__ == '__main__':
    zw = ZkyWarn()
    res = zw.run()  # year=2023
    print(res)  # > [ZkyWarnHit(year=2023, ename='TEXTILE RESEARCH JOURNAL', issn=None, level='中')



# -*- coding: utf-8 -*-
# @time: 2024/3/5 16:11
# @author: Dyz
# @file: jcr.py
# @software: PyCharm
from typing import List

from du_aio_tools.schemas import BaseSchemas
from pydantic import validator


def reset_sn(val: str):
    """校验 sn"""
    if val:
        val = val.upper()
    if '**' in val or 'n/a' in val:
        return ''
    return val.strip()


class EsiHit(BaseSchemas):
    esi_date: str
    abbr: str = None
    abbr2: str = None
    category: str = None


class JcrHit(BaseSchemas):
    ename: str
    abbr: str
    year: int
    issn: str = ''
    eissn: str = ''
    yx: str
    edition: List = []
    zone_rank: List = []
    fq: List = []

    @validator('edition')
    def check_edition(cls, val):
        return list(set(val))

    @validator('issn')
    def check_sn(cls, val):
        return reset_sn(val)

    @validator('eissn')
    def check_en(cls, val):
        return reset_sn(val)

    @validator('yx')
    def check_yx(cls, val):
        if val.upper() == 'N/A':
            return 0
        if '<' in val:
            return val.replace('<', '').strip()
        return val or 0



# -*- coding: utf-8 -*-
# @time: 2024/3/6 8:58
# @author: Dyz
# @file: jcr_login.py
# @software: PyCharm
import json
import time

from playwright.async_api import async_playwright

jcr_file = '.jcr_cookies.json'


async def _login(playwright: async_playwright, auth, ws=None) -> dict:
    if ws:
        browser = await playwright.chromium.connect_over_cdp(ws)
    else:
        browser = await playwright.chromium.launch(headless=False)  # 有界面
        # browser = await playwright.chromium.launch()  # 无界面
    print('jcr 登陆中...')
    context = await browser.new_context()
    page = await context.new_page()
    await page.goto("https://jcr.clarivate.com/jcr/home")
    await page.wait_for_selector("input[name=\"email\"]")  # 等待元素出现
    await page.locator("input[name=\"email\"]").fill(auth[0])
    await page.locator("input[name=\"password\"]").fill(auth[1])
    await page.locator("button:has-text(\"Sign in Loading…\")").click()
    time.sleep(10)  # 再等待登录 cookies 加载完成
    cookies = await context.cookies()
    cookies_dict = dict()
    for cookie in cookies:
        cookies_dict[cookie['name']] = cookie['value']
    cookies_sre = '; '.join(item for item in [item["name"] + "=" + item["value"] for item in cookies])
    pid = cookies_dict['PSSID']
    if pid:
        print('jcr 登陆成功')

    jcr_cookies = {"cookies": cookies_sre, "pid": pid}
    with open(jcr_file, 'w', encoding='utf-8') as f:
        json.dump(jcr_cookies, f)

    # ---------------------
    await context.close()
    await browser.close()
    return jcr_cookies


async def login(auth, ws) -> dict:
    """登录jcr, 使用浏览器获取完整cookies"""

    async with async_playwright() as playwright:
        return await _login(playwright, auth, ws)


async def get_cookies(ws=None, reset=False, auth=None) -> dict:
    """加载本地cookies
    ws: 浏览器实例地址
    reset: 重新登陆
    auth = (user, pwd)
    """
    if reset:
        return await login(auth, ws)
    else:
        with open(jcr_file, encoding='utf-8') as f:
            data = f.read()
            if data:
                return json.loads(data)
        return await login(auth, ws)


if __name__ == '__main__':
    import asyncio

    asyncio.run(get_cookies(True))



# -*- coding: utf-8 -*-
# @project: dujournal
# @Author：dyz
# @date：2024/1/16 14:44
from typing import Union

from crossref.restful import Journals
from du_aio_tools.rsa import make_md5

from schema.jouornal import JournalDoiHit


def parse(data: dict) -> Union[None, JournalDoiHit]:
    if data.get('issn-type'):
        item = JournalDoiHit()
        # issn
        for sn in data['issn-type']:
            if sn.get('type') == 'print':
                item.issn = sn['value']
            elif sn.get('type') == 'electronic':
                item.eissn = sn['value']
        item.id = make_md5(item.issn or item.eissn)

        # 发表卷期的年份
        if data.get('breakdowns', {}).get('dois-by-issued-year'):
            year_list = [_[0] for _ in data['breakdowns']['dois-by-issued-year']]
            item.publication_first_year = min(year_list)
            item.publication_end_year = max(year_list)

        item.title = data['title']

        # 主题词
        if data.get('subjects'):
            mesh, tn = [], []
            for ms in data['subjects']:
                mesh.append(ms['name'])
                tn.append(str(ms['ASJC']))
            item.mesh = ';'.join(mesh)
            item.tn = ';'.join(tn)

        return item
    return None


class DoiJournal(Journals):
    CURSOR_AS_ITER_METHOD = True

    def all_jour(self):
        for row in self.all(None):
            yield parse(row)


if __name__ == '__main__':
    jour = DoiJournal()
    res = jour.journal(issn='2277-338X')
    print(parse(res))
    # for i in jour.all_jour():
    #     print(i)



# -*- coding: utf-8 -*-
# @project: dujournal
# @Author：dyz
# @date：2024/1/17 17:21
from datetime import date
from typing import List, Optional, Union

from du_aio_tools.schemas import BaseSchemas


class RelJournal(BaseSchemas):
    """相关期刊"""
    id: str = None
    title: str
    issn: str = None
    type: str


class LetPubHit(BaseSchemas):
    title: Union[None, str] = None
    issn: Union[None, str] = None
    hit: Union[None, str] = None  # 命中率
    introduction: Union[None, str] = None  # 审稿周期
    cycle: Union[None, str] = None  # 审稿周期
    delivery_link: Union[None, str] = None  # 投稿链接
    website_link: Union[None, str] = None  # 官网链接
    author_notice: Union[None, str] = None  # 作者须知
    p_number: Union[None, str] = None  # 出版量
    self_quote: Union[None, str] = None  # 自引率
    url: Union[None, str] = None  # 网址


class JournalDoiHit(BaseSchemas):
    id: str = ''
    title: str = ''
    # 开始出版年
    publication_first_year: Union[None, str] = None
    # 最后出版年
    publication_end_year: Union[None, str] = None
    mesh: Union[None, str] = None
    tn: Union[None, str] = None
    issn: Union[None, str] = None
    eissn: Union[None, str] = None


class JournalHit(BaseSchemas):
    id: str = ''
    title: Union[None, str] = None
    date_created: Union[None, date] = None
    date_revised: Union[None, date] = None
    date_authorized: Union[None, date] = None
    date_completed: Union[None, date] = None
    date_revised_major: Union[None, date] = None

    title_alternate: Union[None, str] = None
    title_related: List[RelJournal] = None
    abbr: Union[None, str] = None

    note: List[str] = None
    language: List[str] = None
    frequency: str = None

    # 出版类型
    publication_type: List[str] = None
    # 出版信息
    country: Union[None, str] = None
    place_code: Union[None, str] = None
    # 开始出版年
    publication_first_year: Union[None, str] = None
    # 最后出版年
    publication_end_year: Union[None, str] = None

    index_list: Union[List[str], None] = None

    elink: Union[List[str], None] = None
    issn: Union[None, str] = None
    eissn: Union[None, str] = None
    lissn: Union[None, str] = None

    mesh: Union[None, List] = None



# -*- coding: utf-8 -*-
# @time: 2024/3/4 11:03
# @author: Dyz
# @file: parse_journal.py
# @software: PyCharm
from datetime import date
from typing import List

from parsel import Selector
from schema.jouornal import JournalHit, RelJournal

JOURNAL_INDEX = ["MEDLINE", "PubMed", "PMC"]


def parse_date(doc):
    if doc:
        year = int(doc.xpath('./Year/text()').get())
        month = int(doc.xpath('./Month/text()').get())
        day = int(doc.xpath('./Day/text()').get())
        if year:
            return date(year=year, month=month, day=day)
    return None


class ParseJournal:
    def __init__(self, xml: str = None, doc=None):
        if xml:
            doc = Selector(xml, type='xml')
            self.doc = doc.xpath('/NLMCatalogRecordSet/NLMCatalogRecord')
        else:
            self.doc = doc
        self.data = JournalHit(id=self.doc.xpath('./NlmUniqueID/text()').get())

    def parse_date(self):
        """解析时间相关数据"""
        self.data.date_created = parse_date(self.doc.xpath('./DateCreated'))
        self.data.date_revised = parse_date(self.doc.xpath('./DateRevised'))
        self.data.date_authorized = parse_date(self.doc.xpath('./DateAuthorized'))
        self.data.date_completed = parse_date(self.doc.xpath('./DateCompleted'))
        self.data.date_revised_major = parse_date(self.doc.xpath('./DateRevisedMajor'))

    def parse_publication(self):
        """出版物类型"""
        _data = []
        pt_list = self.doc.xpath('./PublicationTypeList')
        for row in pt_list:
            text = row.xpath('./text()').get('').strip()
            if text:
                _data.append(text)
        self.data.publication_type = _data

    def parse_publication_info(self):
        """出版信息"""
        _data = []
        p_info_doc = self.doc.xpath('./PublicationInfo')
        self.data.country = p_info_doc.xpath('./Country/text()').get('').strip() or None
        self.data.place_code = p_info_doc.xpath('./PlaceCode/text()').get('').strip() or None
        self.data.publication_first_year = p_info_doc.xpath('./PublicationFirstYear/text()').get('').strip() or None
        self.data.publication_end_year = p_info_doc.xpath('./PublicationEndYear/text()').get('').strip() or None
        self.data.frequency = p_info_doc.xpath('./Frequency/text()').get('').strip() or None

    def parse_index(self):
        """解析索引"""
        _data = []
        index_list = self.doc.xpath('./IndexingSourceList/IndexingSource')
        for row in index_list:
            text = row.xpath('./IndexingSourceName/text()').get('').strip()
            if text in JOURNAL_INDEX:
                if text == 'MEDLINE':
                    status = row.xpath('./IndexingSourceName/@IndexingStatus').get('').strip()
                    if status == 'Currently-indexed':
                        _data.append(text)
                else:
                    _data.append(text)
        self.data.index_list = _data

    def mash(self):
        """解析期刊中的主题词"""
        mesh_list = []
        mesh_doc = self.doc.xpath('./MeshHeadingList/MeshHeading')
        for row in mesh_doc:
            desc = row.xpath('./DescriptorName/text()').get('').strip()
            q = row.xpath('./QualifierName/text()').get('').strip()
            if q:
                desc += f'/{q}'
            mesh_list.append(desc)
        self.data.mesh = mesh_list

    def parse_language(self):
        """解析语言"""
        _data = []
        note_list = self.doc.xpath('./Language')
        for row in note_list:
            lang_type = row.xpath('./@LangType').get('')
            if lang_type == 'Primary':
                text = row.xpath('./text()').get('').strip()
                if text:
                    _data.append(text)
        self.data.language = _data

    def parse_title(self):
        """解析标题相关"""
        _data = []
        self.data.title = self.doc.xpath('./TitleMain/Title/text()').get('').strip()
        self.data.title_alternate = self.doc.xpath('./TitleAlternate/Title/text()').get('').strip()
        self.data.title_alternate = self.doc.xpath('./TitleAlternate/Title/text()').get('').strip()
        self.data.abbr = self.doc.xpath('./MedlineTA/text()').get('').strip()

    def parse_note(self):
        """解析 Note"""
        _data = []
        note_list = self.doc.xpath('./GeneralNote')
        for row in note_list:
            text = row.xpath('./text()').get('').strip()
            if text:
                _data.append(text)
        self.data.note = _data

    def parse_link(self):
        _data = []
        link_list = self.doc.xpath('./ELocationList/ELocation')
        for row in link_list:
            text = row.xpath('./ELocationID/text()').get('').strip()
            if text:
                _data.append(text)
        self.data.elink = _data

    def parse_issn(self):
        issn_list = self.doc.xpath('./ISSN')
        for row in issn_list:
            text = row.xpath('./text()').get('').strip()
            issn_type = row.xpath('./@IssnType').get('').strip()
            if issn_type == 'Print':
                self.data.issn = text
            elif issn_type == 'Electronic':
                self.data.issn = text
        self.data.lissn = self.doc.xpath('./ISSNLinking/text()').get('').strip()

    def parse_r_jour(self):
        """解析相关期刊"""
        _data_list = []
        rj_list = self.doc.xpath('./TitleRelated')
        for row in rj_list:
            ty = row.xpath('./@TitleType').get('').strip() or None
            ti = row.xpath('./Title/text()').get('').strip() or None
            id_ = row.xpath('./RecordID/text()').get('').strip() or None
            issn = row.xpath('./ISSN/text()').get('').strip() or None
            _data_list.append(RelJournal(title=ti, id=id_, issn=issn, type=ty))
        self.data.title_related = _data_list

    def parse(self) -> JournalHit:
        self.parse_date()
        self.parse_publication()
        self.parse_publication_info()
        self.parse_index()
        self.mash()
        self.parse_language()
        self.parse_title()
        self.parse_note()
        self.parse_link()
        self.parse_issn()
        self.parse_r_jour()
        return self.data


class ParseJournals:
    def __init__(self, xml: str):
        self.doc = Selector(xml, type='xml')
        self.list = self.doc.xpath('/NLMCatalogRecordSet/NLMCatalogRecord')

    def parse(self) -> List[JournalHit]:
        data_list = []
        for doc in self.list:
            parse = ParseJournal(doc=doc)
            data_list.append(parse.parse())
        return data_list



# -*- coding: utf-8 -*-
# @time: 2024/2/26 11:13
# @author: Dyz
# @file: zky.py
# @software: PyCharm
from typing import Union

from du_aio_tools.schemas import BaseSchemas


class ZkyWarnHit(BaseSchemas):
    year: int  # 年分
    ename: str  # 刊名
    issn: Union[str, None] = None  # issn
    level: str  # 预警级别/预警原因


class ZkyHit(BaseSchemas):
    year: int
    ename: str
    abbr: str
    issn: str = None
    zky_big: list = None
    zky_small: list = None
    data: dict = None



# -*- coding: utf-8 -*-
# @time: 2024/1/26 9:46
# @author: Dyz
# @file: sjr.py
# @software: PyCharm
# 检索url % issn
import re
import logging
from typing import List, Union

import httpx
import requests
import pandas as pd
from pathlib import Path
from parsel import Selector

from schema.sjr import SjrHit, SjrRow

logger = logging.getLogger('dujournal.sjr')

title_dict = {
    'Rank': 'rank',  # 排名
    'Sourceid': 'sid',  # sjr id
    'Title': 'ename',  # 刊名
    'Type': 'type',  # 类型： 期刊/?
    'Issn': 'issn',  # 期刊
    'SJR': 'sjr',  # sjr
    'SJR Best Quartile': 'fq',  # sjr fq
    'H index': 'hix',  # hix
    'Country': 'country',  # 国家
    'Region': 'region',  # 地区
    'Publisher': 'publisher',  # 出版商
}


def new_data(row):
    item = {}
    for k, v in row.items():
        if k in title_dict:
            item[title_dict[k]] = v
    return item


class Sjr:
    def __init__(self, path: Path):
        # https://www.scimagojr.com/journalrank.php?out=xls
        self.download_url = 'https://www.scimagojr.com/journalrank.php'
        self.info_url = 'https://www.scimagojr.com/journalsearch.php'
        self.path = path
        self.start_year = 1999

        self.end_year = self.max_year()

    def max_year(self) -> int:
        res = requests.get(self.download_url)
        doc = Selector(res.text)
        max_year = doc.css('#rankingcontrols > div:nth-child(5) > ul > li:nth-child(1) > a::text').get('')
        return int(max_year)

    def query(self, params):
        res = requests.get(self.info_url, params=params)
        doc = Selector(text=res.text)
        res = doc.css("div.pagination ::text").get('').strip()  # 1 - 1 of 1
        number = res.split('of ')[1]
        if number == '1':
            href = doc.css('div.search_results > a ::attr(href)').get('').strip()
            return re.findall('q=(.*?)&', href)[0]
        return False

    def info(self, sid):
        params = {"q": sid, "tip": 'sid', }
        res = requests.get(self.info_url, params=params)
        doc = Selector(res.text)
        ty = doc.xpath('//h2[text()="Publication type"]/following::p[1]/text()').get('').strip()
        issn = doc.xpath('//h2[text()="ISSN"]/following::p[1]/text()').get('').strip()
        ename = doc.xpath('//h1/text()').get('').strip()
        country = doc.xpath('//h2[text()="Country"]/following::p[1]/a/text()').get('').strip()
        hix = doc.css('p.hindexnumber::text').get('').strip()
        publisher = ''
        year = 0
        sjr_list = doc.xpath('//div[@class="cell1x1 dynamiccell"][1]//tr')[1:]
        sjr_text = sjr_list.xpath('.//text()').get('')
        data = []
        sjr = None
        if sjr_text:  # 判断是否有SJR数据
            for td in sjr_list:
                year = td.xpath("./td[1]/text()").get()
                sjr = float(td.xpath("./td[2]/text()").get())
                row = SjrRow(year=int(year), sjr=sjr)
                data.append(row)
        row = SjrHit(
            hix=int(hix),
            issn=issn,
            ename=ename,
            country=country,
            year=year,
            sjr=sjr,
            sjrs=data,
            publisher=publisher,
            sid=sid, type=ty, rank=0)
        return row

    def get_sjr(self, issn=None, eissn=None) -> Union[SjrHit, None]:
        """获取SJR数据"""
        params = {"q": issn.replace('-', '')}
        sid = self.query(params)
        if not sid and eissn:
            params = {"q": issn.replace('-', '')}
            sid = self.query(params)
        if sid:
            return self.info(sid)
        return None

    def get_year_all(self, year, file) -> Path:
        params = {"year": year, "out": "xls"}
        res = requests.get(self.download_url, params=params, stream=True)

        abs_file = Path(self.path, file)
        with open(abs_file, 'wb') as f:
            for chunk in res.iter_content(chunk_size=1024):
                f.write(chunk)
        return abs_file

    def all(self, delete=False) -> SjrHit:
        """
        :param delete: 读取之后是否删除文件，默认不删除
        :return: SjrHit
        """
        for year in range(self.start_year, self.end_year + 1):
            file = Path(f'sjr_{year}.csv')
            if not file.exists():
                file = self.get_year_all(year, file)
            df = pd.read_csv(file, sep=';', iterator=True, chunksize=50, keep_default_na=False)
            # 逐行处理数据
            for chunk in df:
                chunk_list = chunk.to_dict('records')
                for row in chunk_list:
                    data = new_data(row)
                    try:
                        yield SjrHit(**data, year=year)
                    except Exception as e:
                        logger.error(f'数据异常：{e}-{data}')
            if delete:
                file.unlink()  # 删除文件


if __name__ == '__main__':
    path = Path(__file__).parent
    sjr_obj = Sjr(path)
    print(sjr_obj.max_year())
    res = sjr_obj.get_sjr('1004-9649')
    print(res)
    print(res.load_sjrs())
    # for i in sjr_obj.all():
    #     print(i)
    #     input('aa')



# -*- coding: utf-8 -*-
# @project: dujournal
# @Author：dyz
# @date：2024/1/16 14:42
from typing import List

from Bio import Entrez

from parse_xml.parse_journal import ParseJournals


class PubMed:
    def __init__(self):
        self.db = 'nlmcatalog'

        # {"11":"NCBI", "5":"护理刊","4":"MedLine","3":"CORE","2":"PMC","1":"PubMed"}
        self.category = {
            "PUB": "ncbijournals[All Fields]",  # 所有刊检索式 11
            "NU": '("nursing"[Subheading] OR  "nursing"[All Fields] OR "nursing"[MeSH Terms]  OR "nursing"[All Fields] '
                  'OR "breast feeding"[MeSH Terms]  OR  ("breast"[All Fields] AND "feeding"[All Fields]) '
                  'OR breast feeding"[All Fields] OR Nursing[All Fields]) AND ncbijournals[All Fields]',  # 护理刊检索式
            "MD": "currentlyindexed[All]",  # MedLine刊检索式
            # "CR": "jsubsetaim[All Fields]",  # CORE刊检索式, 已经被移除
            "PA": "journalspmc[All Fields]",  # PMC刊检索式
            "PJ": "nlmcatalog pubmed[subset]",  # PubMed刊子集检索式
        }

    def all_ids(self, limit=9998) -> dict:
        """获取所有id"""
        for k, v in self.category.items():
            handle = Entrez.esearch(db=self.db, term=v, retmax=1)
            record = Entrez.read(handle)
            count = int(record['Count'])
            if count > limit:
                for i in range(0, count, limit):
                    handle = Entrez.esearch(db=self.db, term=v, retstart=i, retmax=limit)
                    record = Entrez.read(handle)
                    yield k, record['IdList']
            else:
                handle = Entrez.esearch(db=self.db, term=v, retmax=limit)
                record = Entrez.read(handle)
                yield k, record['IdList']

    def query(self, sn=None, uid=None, db=None):
        """查找PubMed 期刊
            sn 之查找 issn 相关的期刊
            uid 可以 传入 字符串/列表
        """
        db = db or self.db
        if uid:
            if isinstance(uid, list):
                count = len(uid)
            else:
                count = 10
            handle = Entrez.efetch(db=db, id=uid, retmax=count, retmode='xml')
            record = handle.read().decode('utf-8')
            pj = ParseJournals(record)
            handle.close()
            return pj.parse()
        else:
            handle = Entrez.esearch(db=db, term=f'{sn}[All Fields]', retmax=10)
            record = Entrez.read(handle)
            count = int(record['Count'])
            if count >= 1:
                handle = Entrez.efetch(db=db, id=record['IdList'], retmax=count, retmode='xml')
                record = handle.read().decode('utf-8')
                pj = ParseJournals(record)
                handle.close()
                return pj.parse()

    def day_update(self):
        """日更新"""
        # todo


if __name__ == '__main__':
    pub = PubMed()
    Entrez.email = 'xcy@xcy.club'
    Entrez.api_key = '8881d9d0222b33048a5d4a21d4b49ee56f08'
    # data = {}
    # for (k, ids) in pub.all_ids():
    #     if not data.get(k):
    #         data[k] = []
    #     data[k] += ids
    # print(len(data['PUB']))
    # print(len(set(data['PUB'])))

    res = pub.query('2277-338X')
    print(res)



# -*- coding: utf-8 -*-
# @time: 2024/3/5 14:35
# @author: Dyz
# @file: jcr_api.py
# @software: PyCharm
# JCR 提供标准API 需要订阅
# app Subscribe https://developer.clarivate.com/applications
# api docs see https://api.clarivate.com/swagger-ui/?url=https://developer.clarivate.com/apis/wos-journal/swagger



# -*- coding: utf-8 -*-
# @project: dujournal
# @Author：dyz
# @date：2024/1/16 14:42
from datetime import datetime, date
from typing import List

from parsel import Selector
from du_aio_tools.base_spider import AioSpider
from tqdm import tqdm

from jcr.jcr_login import get_cookies
from schema.jcr import JcrHit


class JcrSpider(AioSpider):
    def __init__(self, user=None, pwd=None, cookies=None, headers=None):
        super().__init__(headers=headers)
        self.update_url = 'https://jcr.help.clarivate.com/Content/data-updates.htm'  # 更新公告
        self.search_url = 'https://jcr.clarivate.com/api/jcr3/bwjournal/v1/search-result'  # 检索链接
        self.user = user
        self.pwd = pwd
        if not self.headers:
            self.headers = {"Connection": "keep-alive", "Content-Type": "application/json",
                            "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 "
                                          "(KHTML, like Gecko) Chrome/91.0.4472.124 Safari/537.36"}
        self.cookies = cookies
        self.login_url = 'https://jcr/login'
        self.year = 0
        self.limit = 200
        self.ed_list = ["SCIE", "ESCI", "SSCI", "AHCI"]
        self.query_data = {"journalFilterParameters": {"query": "", "journals": [], "categories": [], "publishers": [],
                                                       "countryRegions": [], "citationIndexes": self.ed_list,
                                                       "jcrYear": self.get_year(), "categorySchema": "WOS",
                                                       "openAccess": "N",
                                                       "jifQuartiles": [], "jifRanges": [], "jifPercentileRanges": [],
                                                       "jciRanges": [], "oaRanges": [], "issnJ20s": []},
                           "retrievalParameters": {"start": 1, "count": self.limit, "sortBy": "jif2019",
                                                   "sortOrder": "DESC"}}

    async def login(self, reset=False, auth=None, ws=None):
        """登录 JCR"""
        cookies = await get_cookies(reset=reset, auth=auth, ws=ws)
        self.headers['Cookie'] = cookies.get("cookies")
        self.headers['x-1p-inc-sid'] = cookies.get("pid")

    def get_year(self):
        return self.year

    async def get_jcr_now_year(self) -> date:
        """获取JCR现在的最新年份"""
        resp = await self.aio_get(self.update_url)
        doc = Selector(resp.text)
        div_list = doc.css('#mc-main-content > div')
        date_list = []
        for div in div_list:
            # October 18, 2023
            text = div.xpath('./span/a/text()').get('').strip()
            date_obj = datetime.strptime(text, '%B %d, %Y').date()
            # 打印年份
            date_list.append(date_obj)
        max_date = max(date_list)
        self.year = max_date.year - 1
        self.query_data['journalFilterParameters']['jcrYear'] = self.year
        return max_date

    def responses(self, resp, **kwargs):
        """可以再次处理相关异常"""
        if self.search_url in str(resp.url):
            if resp.status_code >= 400:
                return None
            if not resp.json():
                return None
            return resp.json()
        return resp

    async def is_login(self, year=2022) -> bool:
        """测试登录是否存在
            test_year: 2022
        """
        await self.login()
        self.year = year
        self.query_data['journalFilterParameters']['jcrYear'] = year
        data = await self.aio_post(self.search_url, json=self.query_data,
                                   headers=self.headers)

        if data:
            return True
        return False

    async def all(self, year: int = None):
        """获取所有数据
        year: 这一年的数据，默认获取最新一年的数据
        """
        if year:
            self.year = year
        else:
            now_date = await self.get_jcr_now_year()
        data_list = await self.aio_post(self.search_url, json=self.query_data,
                                        headers=self.headers)
        total = data_list['totalCount']
        print("total:", total)
        for row in data_list['data']:
            yield self.parse(row)
        for start in tqdm(range(201, total + 1, self.limit)):
            json_data = self.query_data.copy()
            json_data['retrievalParameters']['start'] = start
            data_list = await self.aio_post(self.search_url, json=self.query_data,
                                            headers=self.headers, cookies=self.cookies)
            for row in data_list['data']:
                yield self.parse(row)
            # start_id += self.limit
            # if start_id > total:
            #     break

    def parse(self, data) -> JcrHit:
        """解析数据"""
        item = JcrHit(
            ename=data['journalName'],
            abbr=data['abbrJournal'],
            year=self.year,
            issn=data['issn'],
            eissn=data['eissn'],
            yx=data['jif2019'],
        )
        return self.get_ranks(data, item)

    def get_ranks(self, data, item) -> JcrHit:
        edition = []
        item.zone_rank = []
        item.fq = []
        data_list = data.get('categoryQuartiles', [])

        for row in data_list:
            category, ed = row['category'].split(' - ')
            rank = dict(year=self.year, categoryName=category, edition=ed, jifQuartile=row['quartile'],
                        jifPercentile=row['jifPercentile'], jciPercentile=row['jciPercentile'],
                        jifRank=row['jifRank'], jciRank=row['jciRank'])

            fq = dict(year=self.year, category=category, jifRank=row['jifRank'], edition=ed,
                      quartile=row['quartile'], rank=row['jifRank'], )
            item.zone_rank.append(rank)
            edition.append(ed)
            item.fq.append(fq)
        item.edition = edition
        return item

    async def test(self):
        a = 0
        # total: 21762
        async for row in self.all():
            # print(row)
            a += 1
        print(a)  # 21762


if __name__ == '__main__':
    import asyncio

    jcr = JcrSpider()
    res = asyncio.run(jcr.is_login())



# -*- coding: utf-8 -*-
# @project: dujournal
# @Author：dyz
# @date：2024/1/16 14:41
from datetime import datetime

from du_aio_tools.base_spider import AioSpider

from schema.zky import ZkyHit


def get_now_year():
    """获取当前的年份"""
    return datetime.now().year


class Zky(AioSpider):
    """
    中科院数据
    会返回三种状态
        1. null > http 200
        2. { "Message": "出现错误。" } > http 500
        3. {'Title': 'Urogynecology', 'AbbrTitle': 'UROGYNECOLOGY', 'ISSN': '2771-1897', 'Review': False, 'Year': 2023,
        'ZKY': [{'Top': False, 'Name': '医学', 'Section': 3}],
        'JCR': [{'NameCN': '妇产科学', 'Name': 'OBSTETRICS & GYNECOLOGY', 'Section': 3}], 'Indicator': None} > http 200
    """

    def __init__(self):
        super().__init__()
        self.start_year = 2005  # 中科院数据开始的年份
        self.start_v2_year = 2020  # 升级版开始的年份
        self.ZKY_QUERY = 'https://webapi.fenqubiao.com/api/journal'

    def get_params(self, abbr, year):
        params = {"abbr": abbr, "year": year}
        if not params["year"]:
            params['year'] = get_now_year()
        if params['year'] >= self.start_v2_year:
            params['version'] = 2
        return params

    def responses(self, resp, **kwargs):
        """可以再次处理相关异常"""
        if resp.status_code == 500:
            return None
        if not resp.json():
            return None
        return resp.json()

    async def query(self, abbr, year=None):
        """查询中科院数据"""
        params = self.get_params(abbr, year)
        data = await self.aio_get(self.ZKY_QUERY, params=params)
        if not data and not year:
            params['year'] -= 1
            data = await self.aio_get(self.ZKY_QUERY, params=params)
        if data:
            return ZkyHit(
                ename=data["Title"],
                abbr=data["AbbrTitle"],
                issn=data["ISSN"],
                year=data["Year"],
                zky_big=data["ZKY"],
                zky_small=data["JCR"],
                data=data
            )


if __name__ == '__main__':
    zky = Zky()
    import asyncio

    res = asyncio.run(zky.query('UROGYNECOLOGY', year=2024))
    print(res)



# -*- coding: utf-8 -*-
# @time: 2024/3/5 14:28
# @author: Dyz
# @file: parse_mesh.py
# @software: PyCharm
from pathlib import Path
from typing import List

from parsel import Selector

from schema.mesh import MeshHit


def row_supp(xml) -> MeshHit:
    """解析补充概念"""
    doc = Selector(xml, type='xml')
    uid = doc.xpath('.//SupplementalRecordUI/text()').get('').strip()
    name = doc.xpath('.//SupplementalRecordName/String/text()').get('').strip()
    note = doc.xpath('.//Note/text()').get('').strip()
    ey = doc.xpath('.//TermList//String/text()').getall()
    hm_list = doc.xpath('.//HeadingMappedToList//DescriptorReferredTo')
    hm = []
    for h_ in hm_list:
        hm_name = h_.xpath('.//DescriptorName/String/text()').get('').strip()
        hm_uid = h_.xpath('.//DescriptorUI/text()').get('').strip()
        if hm_name:
            hm.append(f'{hm_uid}@@{hm_name}')

    if ey and ey[0] == name:
        ey.pop(0)
    return MeshHit(
        uid=uid,
        name=name,
        note=note,
        hm=hm,
        ey=ey,
        tp='SC'
    )


def row_qual(xml) -> MeshHit:
    """解析副主题词"""
    doc = Selector(xml, type='xml')
    uid = doc.xpath('.//QualifierUI/text()').get('').strip()
    name = doc.xpath('.//QualifierName/String/text()').get('').strip()
    note = doc.xpath('.//ConceptList/Concept/ScopeNote/text()').get('').strip()
    tn = doc.xpath('.//TreeNumberList/TreeNumber/text()').getall()
    abbr = doc.xpath('.//Abbreviation/text()').get('').strip()
    return MeshHit(
        uid=uid,
        name=name,
        note=note,
        tn=tn,
        abbr=abbr,
        tp='QU',
    )


def row_desc(xml) -> MeshHit:
    """解析主题词"""
    doc = Selector(xml, type='xml')
    uid = doc.xpath('.//DescriptorUI/text()').get('').strip()
    name = doc.xpath('.//DescriptorName/String/text()').get('').strip()
    note = doc.xpath('.//ConceptList/Concept/ScopeNote/text()').get('').strip()
    tn = doc.xpath('.//TreeNumberList/TreeNumber/text()').getall()
    ey = doc.xpath('.//TermList//String/text()').getall()
    if ey and ey[0] == name:
        ey.pop(0)

    return MeshHit(
        uid=uid,
        name=name,
        note=note,
        tn=tn,
        tp='D',
        ey=ey
    )


class ParseMeshList:
    def __init__(self, file: Path = None, text: str = None):
        """
        :param file: 解析xml文件
        :param text: 解析text文本网页源码
        """
        self.file = file
        self.text = text

    def tp(self) -> str:
        """判断数类型"""
        import itertools
        text = ''
        with open(self.file, encoding='utf-8') as f:
            for line in itertools.islice(f, 4):
                if line:
                    text = line
        return text

    def parse(self):
        text = self.tp()
        if text.startswith('<Desc'):
            for row in self.parse_desc():
                yield row
        elif text.startswith('<Supp'):
            for row in self.parse_supp():
                yield row
        elif text.startswith('<Qual'):
            for row in self.parse_qual():
                yield row

    def read_line(self, text) -> str:
        """数据按行读取"""
        xml = ''
        with open(self.file, 'r', encoding='utf-8') as f:
            for line in f:
                # 每条数据的开头
                if line.startswith(text):
                    if xml:  # 如果有上一条就返回后赋予新的
                        yield xml
                        xml = line
                        continue
                    else:  # 没有上一条就直接加在一起
                        xml += line
                        continue
                if xml:
                    xml += line
            if xml:
                yield xml

    def parse_desc(self) -> List[MeshHit]:
        """解析主题词"""
        for xml in self.read_line(text='<DescriptorRecord DescriptorClass'):
            yield row_desc(xml)

    def parse_supp(self):
        """解析补充概念"""
        for xml in self.read_line(text='<SupplementalRecord SCRClass'):
            yield row_supp(xml)

    def parse_qual(self):
        """解析补充概念"""
        for xml in self.read_line(text='<QualifierRecord>'):
            yield row_qual(xml)

    def test(self):
        for row in self.parse():
            print(row)


if __name__ == '__main__':
    file = Path(r'D:\work\pypi_hub\dujournal\mesh\qual2024.xml')
    mesh = ParseMeshList(file)
    print(mesh.test())



# -*- coding: utf-8 -*-
# @time: 2023/3/7 10:26
# @author: Dyz
# @file: letpub.py
# @software: PyCharm
# 投稿相关数据经过对比，梅斯数据效果不如 letpub
# 该网站数据收录只有SCI期刊数据
import re
import time

import httpx
from du_aio_tools.base_spider import AioSpider
from parsel import Selector

from schema.jouornal import LetPubHit


class Spider(AioSpider):
    def __init__(self, wait_fixed=5000):
        super().__init__()
        self.url = 'https://www.letpub.com.cn/index.php'
        self.info_url = 'http://www.letpub.com.cn/index.php?journalid={}&page=journalapp&view=detail'
        self.wait_html = ['您刷新页面的速度过快', '请求期刊页面数据过于频繁', '温馨提示：您使用的IP地址']
        self.wait_fixed = wait_fixed
        # self.headers = {
        #     'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.7',
        #     'Accept-Language': 'zh-CN,zh;q=0.9,en;q=0.8,en-GB;q=0.7,en-US;q=0.6',
        #     'Cache-Control': 'max-age=0',
        #     'DNT': '1',
        #     'Origin': 'http://www.letpub.com.cn',
        #     'Referer': 'http://www.letpub.com.cn/index.php?page=journalapp&view=search',
        #     'Upgrade-Insecure-Requests': '1',
        #     'User-Agent': self.get_ua()}
        # self.client = httpx.AsyncClient(limits=self.limits, verify=False)  # 异步

    def responses(self, resp, **kwargs):
        """letpub 有ip访问速度的限制"""
        html = resp.text
        for text in self.wait_html:
            if text in html:
                time.sleep(self.wait_fixed)
                raise
        return resp

    def get_params(self, sn, abbr):
        params = {
            'page': 'journalapp',
            'view': 'search',
        }

        data = {
            'searchname': abbr or '',
            'searchissn': sn or '',
            'searchfield': '',
            'searchimpactlow': '',
            'searchimpacthigh': '',
            'searchscitype': '',
            'view': 'search',
            'searchcategory1': '',
            'searchcategory2': '',
            'searchjcrkind': '',
            'searchopenaccess': '',
            'searchsort': 'relevance',
        }
        return params, data

    async def query(self, sn=None, abbr=None):
        """查找数据 """
        if not sn and not abbr:
            raise
        params, data = self.get_params(sn, abbr)
        res = await self.aio_post(self.url, params=params, json=data)
        html = res.text
        jid = re.findall('journalid=(\d+)&page=journalapp', html)
        if not jid:
            jid = re.findall('journalid=(\d+)&page=journalapp', html)
        if jid:
            url = self.info_url.format(jid[0])
            resp = await self.aio_get(url, headers=self.headers)
            item = LetPubHit(url=url, issn=sn, title=abbr)
            return self.parse(resp.text, item)

    def parse(self, html, item) -> LetPubHit:
        doc = Selector(html)
        item.introduction = doc.xpath("string(//td[contains(text(),'期刊简介')]/following-sibling::td)").get(
            '').strip()

        item.website_link = doc.xpath("string(//td[contains(text(),'期刊官方网站')]/following-sibling::td)").get(
            '').strip()

        item.delivery_link = doc.xpath(
            "string(//td[contains(text(),'期刊投稿网址')]/following-sibling::td)").get('').strip()

        item.author_notice = doc.xpath(
            "string(//td[contains(text(),'作者指南网址')]/following-sibling::td)").get('').strip()

        self_quote = doc.xpath("string(//td[contains(text(),'自引率')]/following-sibling::td)").get('').strip()
        if '%' in self_quote:
            self_quote = self_quote.split('%', 1)[0]
            if self_quote.strip():
                item.self_quote = self_quote + '%'  # 自引率

        item.hit = doc.xpath("string(//td[contains(text(),'平均录用比例')]/following-sibling::td)").get(
            '').strip().split('：')[-1]

        item.cycle = doc.xpath("string(//td[contains(text(),'平均审稿速度')]/following-sibling::td)").get(
            '').strip().split('：')[-1]
        item.p_number = doc.xpath("string(//td[contains(text(),'年文章数')]/following-sibling::td)").get(
            '').strip().split('注册')[0].split('点击查看')[0].strip()
        return item

    async def test(self, issn='0393-974X'):
        data = await self.query(issn)
        print(data)


if __name__ == '__main__':
    import asyncio

    let_pub_obj = Spider()

    asyncio.run(let_pub_obj.test(issn='0066-4154'))



# -*- coding: utf-8 -*-
# @Time    : 2021/9/1 10:05
# @Author  : DYZ
# @Software: PyCharm

from pathlib import Path

from setuptools import find_packages, setup

# 提供一些有用的信息
NAME = 'dujournal'  # Python 包的名称，即在 pip install 时后面跟的包名
VERSION = '0.0.2'  # 包的版本，每次上传到 PyPI 都需要改变这个版本号，否则只会往存储空间增加新内容，无法达到预期
DESCRIPTION = "Some common websites related to journals, such as PubMed, JCR, ZKY, SJR"  # 关于该包的剪短描述

readme = Path('README.md')

if readme.exists():  # 如果需要，可以加入一段较长的描述，比如读取 README.md，该段长描述会直接显示在 PyPI 的页面上
    with open(readme, encoding='utf-8') as f:
        LONG_DESCRIPTION = f.read()
else:
    LONG_DESCRIPTION = DESCRIPTION
AUTHOR = 'DYZ'  # 留下大名
AUTHOR_EMAIL = 'duyuchau@gmail.com'  # 留下邮箱
LICENSE = 'MIT'  # 定义合适自己的许可证，实在不知道，那就 MIT 吧
PLATFORMS = [  # 支持的平台，如果所有平台都支持，可以填 all
    'all',
]

ALL = [  # 很多时候，我自己写的包都要依赖第三方，所以可以把依赖包定义在这里，这样的 pip install 自己包的时候，顺便把这些依赖包都装上了
    'httpx',
    'parsel',
    'requests',
    'parsel',
    'biopython==1.83',
    'tenacity',
    'pydantic~=1.10.2',
    'httpx[http2]',
]
# CONSOLE_SCRIPT = 'my-cmd=my_pkg.my_cmd:main'                  # 如果想在 pip install 之后自动生成一个可执行命令，就靠它了:


# 需要的信息就在 setup() 中加上，不需要的可以不加
setup(
    name=NAME,
    version=VERSION,
    description=(
        DESCRIPTION
    ),
    long_description=LONG_DESCRIPTION,
    long_description_content_type='text/markdown',
    author=AUTHOR,
    author_email=AUTHOR_EMAIL,
    maintainer=AUTHOR,
    maintainer_email=AUTHOR_EMAIL,
    license=LICENSE,
    packages=find_packages(),
    platforms=PLATFORMS,
    install_requires=ALL,
)



# -*- coding: utf-8 -*-
# @project: dujournal
# @Author：dyz
# @date：2024/1/19 10:31
from datetime import date
from typing import List, Optional, Union

from du_aio_tools.schemas import BaseSchemas


class MeshHit(BaseSchemas):
    uid: str  # 主题词ID
    name: str
    note: str  # 英文解释
    tn: List[str] = None  # 树状号 Tree number
    hm: List[str] = None  # 树状号 Tree number
    # ph: str = None # ?
    ey: List[str] = None  # 入口词
    tp: str
    tr: str = None  # MeSH Categories
    aq: str = None  # 副主题词
    abbr: str = ''



# -*- coding: utf-8 -*-
# @time: 2024/3/5 15:09
# @author: Dyz
# @file: esi.py
# @software: PyCharm
import re
import io
from datetime import datetime
from typing import List

import pandas as pd
from parsel import Selector
from du_aio_tools.base_spider import AioSpider

from schema.jcr import EsiHit


class EsiJour(AioSpider):
    """ESI 期刊"""

    def __init__(self):
        super().__init__()
        self.suffix = '.xlsx'
        self.url = 'https://esi.help.clarivate.com/Content/journal-list.htm'
        self.download_url = 'https://esi.help.clarivate.com/Content/'

    async def all(self):
        """获取所有年份的数据"""
        resp = await self.aio_get(self.url)
        async for row in self.parse(resp):
            yield row

    async def parse(self, resp):
        """解析数据"""
        doc = Selector(resp.text)
        href = doc.css('#mc-main-content > p:nth-child(2) > a::attr(href)').get('').strip()
        text = doc.xpath('string(//*[@id="mc-main-content"]/p[2])').get('').strip()
        data = re.findall('The current extraction date for the journal list is (.*?)\.', text, flags=re.S | re.I)
        date_obj = str(datetime.strptime(data[0], '%B %d, %Y').date())
        url = self.download_url + href
        resp = await self.aio_get(url)
        df = pd.read_excel(resp.content, keep_default_na=False)
        for row in df.to_dict('records'):
            abbr = row.get('journal title', '') or row.get('Title20', '')
            abbr2 = row.get('title29', '') or row.get('Title29', '')
            category = row.get('category name', '') or row.get('Category Name', '')
            if category:
                yield EsiHit(esi_date=date_obj, abbr=abbr, abbr2=abbr2, category=category)

    async def test(self):
        async for row in self.all():
            print(row)


if __name__ == '__main__':
    esi = EsiJour()
    import asyncio

    res = asyncio.run(esi.test())



# -*- coding: utf-8 -*-
# @project: dujournal
# @Author：dyz
# @date：2024/1/16 14:42
import re
from pathlib import Path

from du_aio_tools.base_spider import AioSpider
from du_aio_tools.time import timer
from parsel import Selector

from schema.mesh import MeshHit


class Mesh(AioSpider):
    def __init__(self, path: Path):
        super().__init__()
        self.url = 'https://nlmpubs.nlm.nih.gov/projects/mesh/MESH_FILES/xmlmesh/'
        self.text_url = 'https://www.ncbi.nlm.nih.gov/mesh/'
        self.path = path
        self.category = {
            'desc': 'desc',  # 主题词
            'supp': 'supp',  # 补充概念
            'qual': 'qual',  # 副主题词
        }

    async def download_zip(self):
        """下载 zip 数据包"""
        resp = await self.aio_get(self.url)
        doc = Selector(resp.text)
        list_xml_file = doc.xpath('//a[contains(text(),".xml")]')
        for file in list_xml_file:
            name = file.xpath('./text()').get('').strip()
            href = file.xpath('./@href').get('').strip()
            file_path = Path(self.path, name)
            # print(name, href)
            await self.aio_download(self.url + href, file_path)

    async def load_text(self, uid):
        """加载text
        https://www.ncbi.nlm.nih.gov/mesh/?term=D000001&report=Full&format=text
        """
        params = {
            'term': uid,
            'report': 'Full',
            'format': 'text',
        }
        resp = await self.aio_get(self.text_url, params=params)
        text_list = re.findall('<pre>(.*?)</pre>', resp.text, flags=re.M | re.S)
        if text_list:
            return text_list[0]
        return None

    async def parse(self, uid) -> [None, tuple]:
        """解析 text"""
        text = await self.load_text(uid)
        if text:
            tr = self.get_tr(text)
            aq = self.get_aq(text)
            return tr, aq, text
        return None

    def get_tr(self, text):
        """从下载好的网页中提取树状结构
        部分主题词后面 带一个空格加号 ' +'， 匹配主题词id时需要注意
        """
        tr_text = '    ' + text[text.find('All MeSH Categories'):]
        data_list = []
        for line in tr_text.splitlines():
            if line.strip():
                # 用每一行前面的空格数量来判断文字的位置
                index = ((len(line) - len(line.lstrip(' '))) // 4) - 1
                if index == -1:
                    break
                data_list.append((index, line.strip()))
        return data_list

    def get_aq(self, text) -> list:
        """ 获取副主题词 """
        data_list = []
        text_list = text.splitlines()
        sta = False
        for row in text_list:
            if 'Subheadings:' == row.strip():
                sta = True
                continue
            if sta:
                if not row.strip():
                    break
                name = row.strip()
                data_list.append(name)
        return data_list

    async def test(self):
        await self.download_zip()


if __name__ == '__main__':
    path = Path(__file__).parent
    mesh = Mesh(path)
    import asyncio

    # asyncio.run(mesh.test())
    asyncio.run(mesh.parse('D000001'))




# -*- coding: utf-8 -*-
# @time: 2024/3/7 10:02
# @author: Dyz
# @file: sjr.py
# @software: PyCharm
from typing import Union, List

import requests
from du_aio_tools.schemas import BaseSchemas
from parsel import Selector
from pydantic import validator

SJR_QUERY = 'https://www.scimagojr.com/journalsearch.php'


class SjrRow(BaseSchemas):
    year: int
    sjr: Union[float, None] = None


class SjrHit(BaseSchemas):
    sid: str
    year: int
    rank: int
    hix: int = None
    issn: Union[List[str], str, None]
    ename: str
    type: str
    fq: Union[str, None] = None
    sjr: Union[float, None] = None
    country: str = None
    region: str = None
    sjrs: Union[List[SjrRow], None] = None
    publisher: str

    @validator('fq')
    def check_fq(cls, val):
        if not val:
            return None
        if val == '-':
            return None
        return val

    @validator('sjr')
    def check_sjr(cls, val):
        if isinstance(val, float):
            return val
        if ',' in val:
            return float(val.replace(',', '.'))
        return val or None

    @validator('issn')
    def check_issn(cls, val):
        if not val.strip():
            return None
        issn_list = val.split(', ')
        data = []
        for sn in issn_list:
            if len(sn) == 8:
                data.append(sn[:4] + '-' + sn[4:])
        return data

    def load_sjrs(self) -> List[SjrRow]:
        data = []
        params = {
            "q": self.sid,
            "tip": "sid"
        }
        response = requests.get(SJR_QUERY, params=params)
        doc = Selector(response.text)
        response.close()

        sjr_list = doc.xpath('//div[@class="cell1x1 dynamiccell"][1]//tr')[1:]
        sjr = sjr_list.xpath('.//text()').get('')
        if sjr:  # 判断是否有SJR数据
            for td in sjr_list:
                year = td.xpath("./td[1]/text()").get()
                sjr = td.xpath("./td[2]/text()").get()
                row = SjrRow(year=int(year), sjr=float(sjr))
                data.append(row)
        return data
